#!/bin/bash

BASE_APM_PATH="/opt/datadog/apm"
BINARY_INSTALL_PATH="$BASE_APM_PATH/inject"
LIBRARIES_INSTALL_BASE="$BASE_APM_PATH/library"
UNIX_DOMAIN_SOCKET_PATH_DIR="$BINARY_INSTALL_PATH/run"
UNIX_DOMAIN_SOCKET_PATH="$UNIX_DOMAIN_SOCKET_PATH_DIR/apm.socket"
DOGSTATSD_SOCKET_PATH="$UNIX_DOMAIN_SOCKET_PATH_DIR/dsd.socket"
daemon_config_dir="/etc/docker"
daemon_config="/etc/docker/daemon.json"
daemon_config_backup="/etc/docker/daemon.json.bak"

# Root user detection
if [ "$UID" = "0" ]; then
    sudo_cmd=''
else
    sudo_cmd='sudo'
fi

# sudo_cmd_test should be used for checks that do not have side effects (eg in `if` statements)
# unlike sudo_cmd, sudo_cmd_test doesn't alter the return code in dry-run mode
sudo_cmd_test=$sudo_cmd

function exit_error() {
  exit 2
}

function exit_success() {
  exit 0
}

function service_control() {
  action=$1
  service_name=$2


  # Service command discovery. Copied from https://github.com/DataDog/agent-linux-install-script/blob/f8471b2c27c4c7a98f16b851aedb70e9c4c54a1c/install_script.sh.template#LL791-L812C3
  service_cmd="service"
  if [ "$SUSE11" = "yes" ]; then
    service_cmd=$($sudo_cmd which service)
  fi

  # Use /usr/sbin/service by default.
  # Some distros usually include compatibility scripts with Upstart or Systemd. Check with: `command -v service | xargs grep -E "(upstart|systemd)"`
  restart_cmd="$sudo_cmd $service_cmd $service_name restart"
  reload_cmd="$sudo_cmd $service_cmd $service_name reload"

  if [ "$($sudo_cmd ps --no-headers -o comm 1 2>&1)" == "systemd" ] && command -v systemctl 2>&1; then
    # Use systemd if systemctl binary exists and systemd is the init process
    restart_cmd="$sudo_cmd systemctl restart ${service_name}.service"
    reload_cmd="$sudo_cmd systemctl reload ${service_name}.service"
  elif /sbin/init --version 2>&1 | grep -q upstart; then
    # Try to detect Upstart, this works most of the times but still a best effort
    restart_cmd="$sudo_cmd stop $service_name || true ; sleep 2s ; $sudo_cmd start $service_name"
    reload_cmd=$restart_cmd
  fi

  if [ "restart" = "$action" ]; then
    printf "\033[34m* Restarting %s...\n\033[0m\n" "$service_name"

    if ! eval "$restart_cmd"; then
      printf "\033[34m* Restart failed \033[0m\n"
      exit_error
    fi
  else
    printf "\033[34m* Reloading %s...\n\033[0m\n" "$service_name"
    if ! eval "$reload_cmd"; then
      printf "\033[34m* Reload failed or is not available. Trying to restart instead \033[0m\n"
      
      if ! eval "$restart_cmd"; then
        printf "\033[34m* Restart failed \033[0m\n"
        exit_error
      fi
    fi
  fi

}

function enable_dry_run() {
  sudo_cmd_orig="$sudo_cmd"
  sudo_cmd=dry_run_sudo

  eval 'function service_control() { dry_run_service_control $1 $2; }'
  eval 'function exit_success() { dry_run_exit_success; }'
}

function dry_run_sudo() {
  echo "[DRY RUN] $sudo_cmd_orig $*"
  command_executed="true"
}

function dry_run_service_control() {
  echo "[DRY RUN] $*"
}

function dry_run_exit_success() {
  if [ -n "$command_executed" ] ; then
    exit 1
  else
    exit 0
  fi
}

# Parse command line args
while [ $# -gt 0 ]; do
    case $1 in
        -h|--help)
            usage
            exit_success
            ;;
        --dry-run)
            enable_dry_run
            shift
            ;;
        --uninstall)
            uninstall_flag="true"
            shift
            ;;
        --no-config-change)
            no_config_change="true"
            shift
            ;;
        --no-agent-restart)
            no_agent_restart="true"
            shift
            ;;
        --no-agent-config-change)
            no_agent_config_change="true"
            shift
            ;;
        --no-docker-reload)
            no_docker_reload="true"
            shift
            ;;
        *)
            echo -e "ERROR: An unknown flag was passed: ${1}\n"
            usage
            exit_error
            ;;
    esac
done
